'''
logging class for CFchecker
'''
from copy import deepcopy

from errors import MessageLogOverflow

class MessageLog(object):
    '''
    Message logger class for CF checker
    logs the three categories of messages ERROR, WARNING and INFO by invoking the respective method.
    test_id is the ID of the test the log entry is refering to and message is a freestyle message string
    describing the problem found.
    to prevent exhaustive logging to jam all memory, the entries_max contructor value defines the maximum
    number of entries to be saved before a MessageLogOverflow exception is raised.
    a special NOT IMPLEMENTED category is available for recording information about features that have
    not been implemented in the tests. these entries are only saved if the log_not_implemented parameter
    is passed to the contructor, else those entries are silently dropped when not_implemented() is called.
    the log can be retrieved either as a string with one log entry per line or in raw format as it is
    used internally.
    presence of errors or warnings in the log can be checked with the has_errors() and has_warnings() methods.
    '''
    def __init__(self, log_not_implemented=False, max_entries=500):
        self.log = []
        self.log_not_implemented = log_not_implemented
        self.max_entries = min([max_entries, 2000])
    def _add_log_entry(self, logtype, test_id, message, do_check=True):
        self.log.append((logtype, test_id, message))
        if do_check: self.__check_limits()
    def error(self, test_id, message):
        self._add_log_entry('ERROR', test_id, message)
    def warning(self, test_id, message):
        self._add_log_entry('WARNING', test_id, message)
    def info(self, test_id, message):
        self._add_log_entry('INFO', test_id, message)
    def not_implemented(self, test_id, message):
        if not self.log_not_implemented: return
        self._add_log_entry('NOT IMPLEMENTED', test_id, message)
    def has_errors(self):
        return 'ERROR' in [item[0] for item in self.log]
    def has_warnings(self):
        return 'WARNING' in [item[0] for item in self.log]
    def __check_limits(self):
        if len(self.log) >= self.max_entries:
            self._add_log_entry('-', '-', 'maximum number of log entries reached, checking stopped', do_check=False)
            raise MessageLogOverflow(self, 'maximum number of log entries reached')
    def __str__(self):
        ret = ''
        for entry in self.log:
            ret += '%s: %s:\t%s\n' % entry
        # cut last \n if there is something to return
        return ret[:-1] if ret else ''
    def get_raw_log(self):
        return deepcopy(self.log)
    def clear(self):
        '''clear all log entries'''
        self.log = []
