'''
standard name table module
'''

import os

from lxml import etree

from errors import StandardNameTableCorruptError, StandardNameTableNotFoundError

class StandardNameTable(object):
    def __init__(self, filename):
        if not os.path.isfile(filename):
            raise StandardNameTableNotFoundError('XML input file not found')
        parser = etree.XMLParser(remove_blank_text=True)
        self.tree = etree.parse(filename, parser)
        self.root = self.tree.getroot()
        # generate and save a list of all standard names in file
        self.names = [node.get('id') for node in self.root.iter('entry')]
        self.aliases = [node.get('id') for node in self.root.iter('alias')]
        # save version number and last modified date
        version_tag = self.root.find('version_number')
        last_modified_tag = self.root.find('last_modified')
        if version_tag is None or last_modified_tag is None:
            raise StandardNameTableCorruptError('version_number and/or last_modified tags missing')
        self.version = version_tag.text
        self.last_modified = last_modified_tag.text

    def get_version(self):
        return self.version

    def get_last_modified(self):
        return self.last_modified

    def exists(self, standard_name):
        # first search in entry fields, then in alias fields if entry was not found
        return standard_name in self.names or self.is_alias(standard_name)

    def is_alias(self, standard_name):
        return standard_name in self.aliases

    def lookup_alias(self, alias_name):
        if not alias_name in self.aliases:
            raise StandardNameTableNotFoundError('alias name not found in table')
        idx = self.aliases.index(alias_name)
        # find real name for alias
        real_name = self.root.xpath('alias')[idx].xpath('entry_id')
        if len(real_name) != 1:
            raise StandardNameTableCorruptError('not exactly one entry_id tag found for alias %s, standard name table seems to be broken' % alias_name)
        else:
            return real_name[0].text.strip()

    def get_units(self, standard_name):
        if not standard_name in self.names:
            # not found in entries, so look for aliases
            if not self.is_alias(standard_name):
                raise StandardNameTableNotFoundError('standard name not found in table')
            # find real name for alias
            real_name = self.lookup_alias(standard_name)
        else:
            real_name = standard_name
        idx = self.names.index(real_name)
        units = self.root.xpath('entry')[idx].xpath('canonical_units')
        if len(units) == 0:
            raise StandardNameTableCorruptError('canonical_unit tag not found for entry %s' % real_name)
        elif len(units) > 1:
            raise StandardNameTableCorruptError('more than one canonical_unit tag found for entry %s' % real_name)
        return units[0].text.strip()
